@@ -1,13 +1,24 @@
 import asyncio
-from typing import List
+from typing import Callable, List
 
-from pyppeteer.page import Page
+from playwright.async_api import Page, TimeoutError
 from tqdm import tqdm
 
 from tiktokpy.client import Client
+from tiktokpy.utils import unique_dicts_by_key
 from tiktokpy.utils.client import catch_response_and_store, catch_response_info
 from tiktokpy.utils.logger import logger
 
+FEED_LIST_ITEM = 'div[data-e2e="recommend-list-item-container"]'
+USER_FEED_LIST = 'div[data-e2e="user-post-item-list"]'
+USER_FEED_ITEM = f"{USER_FEED_LIST} > div"
+USER_FEED_LAST_ITEM = f"{USER_FEED_ITEM}:last-child"
+FOLLOW_BUTTON = 'button[data-e2e="follow-button"]'
+UNFOLLOW_BUTTON = 'div[class*="DivFollowIcon"]'
+MAIN_WRAPPER = "div[class*=DivThreeColumnContainer],main[class*=MainDetailWrapper]"
+ERROR_TITLE = "main div[class*=ErrorContainer] p"
+SEARCH_USERNAME = 'a[href="/{}"]'
+
 
 class User:
     def __init__(self, client: Client):
@@ -31,14 +42,16 @@ class User:
         await self.client.goto(
             f"/@{username}/video/{video_id}",
             page=page,
-            options={"waitUntil": "networkidle0"},
+            wait_until="networkidle",
         )
 
-        like_selector = ".lazyload-wrapper:first-child .item-action-bar.vertical > .bar-item-wrapper:first-child"  # noqa: E501
-        is_liked = await page.J(f'{like_selector} svg[fill="none"]')
+        like_selector = f'{FEED_LIST_ITEM}:first-child span[data-e2e="like-icon"]'
+        is_liked = await page.query_selector(f"{like_selector} > div > svg")
 
         if is_liked:
             logger.info(f"😏 @{username}'s video {video_id} already liked")
+            await page.close()
+
             return
 
         await page.click(like_selector)
@@ -70,21 +83,23 @@ class User:
         await self.client.goto(
             f"/@{username}/video/{video_id}",
             page=page,
-            options={"waitUntil": "networkidle0"},
+            wait_until="networkidle",
         )
 
-        like_selector = ".lazyload-wrapper:first-child .item-action-bar.vertical > .bar-item-wrapper:first-child"  # noqa: E501
-        is_unliked = await page.J(f'{like_selector} svg[fill="currentColor"]')
+        like_selector = f'{FEED_LIST_ITEM}:first-child span[data-e2e="like-icon"]'
+        is_unliked = not await page.query_selector(f"{like_selector} > div > svg")
 
         if is_unliked:
             logger.info(f"😏 @{username}'s video {video_id} already unliked")
+            await page.close()
+
             return
 
         await page.click(like_selector)
 
-        like_info = await like_info_queue.get()
+        unlike_info = await like_info_queue.get()
 
-        if like_info["status_code"] == 0:
+        if unlike_info["status_code"] == 0:
             logger.info(f"👎 @{username}'s video {video_id} unliked")
         else:
             logger.warning(f"⚠️  @{username}'s video {video_id} probably not unliked")
@@ -109,19 +124,21 @@ class User:
         await self.client.goto(
             f"/@{username.lstrip('@')}",
             page=page,
-            options={"waitUntil": "networkidle0"},
+            wait_until="networkidle",
         )
 
-        follow_title: str = await page.Jeval(
-            ".follow-button",
-            pageFunction="element => element.textContent",
+        follow_title: str = await page.eval_on_selector(
+            FOLLOW_BUTTON,
+            expression="element => element.textContent",
         )
 
         if follow_title.lower() != "follow":
             logger.info(f"😏 {username} already followed")
+            await page.close()
+
             return
 
-        await page.click(".follow-button")
+        await page.click(FOLLOW_BUTTON)
 
         follow_info = await follow_info_queue.get()
 
@@ -150,19 +167,19 @@ class User:
         await self.client.goto(
             f"/@{username.lstrip('@')}",
             page=page,
-            options={"waitUntil": "networkidle0"},
+            wait_until="networkidle",
         )
 
-        follow_title: str = await page.Jeval(
-            ".follow-button",
-            pageFunction="element => element.textContent",
+        follow_title: str = await page.eval_on_selector(
+            FOLLOW_BUTTON,
+            expression="element => element.textContent",
         )
 
         if follow_title.lower() != "following":
             logger.info(f"😏 {username} already unfollowed")
             return
 
-        await page.click(".follow-button")
+        await page.click(UNFOLLOW_BUTTON)
 
         unfollow_info = await unfollow_info_queue.get()
 
@@ -173,10 +190,23 @@ class User:
 
         await page.close()
 
-    async def feed(self, username: str, amount: int):
+    async def feed(self, username: str, amount: int) -> List[dict]:
         page: Page = await self.client.new_page(blocked_resources=["image", "media", "font"])
         logger.debug(f"📨 Request {username} feed")
 
+        _ = await self.client.goto(
+            f"/search/user?q={username.lstrip('@')}",
+            page=page,
+            wait_until="networkidle",
+        )
+        username_selector = SEARCH_USERNAME.format(username)
+
+        is_found_user = await page.query_selector(username_selector)
+
+        if not is_found_user:
+            logger.error(f'❗️ User "{username}" not found')
+            return []
+
         result: List[dict] = []
 
         page.on(
@@ -184,61 +214,87 @@ class User:
             lambda res: asyncio.create_task(catch_response_and_store(res, result)),
         )
 
-        _ = await self.client.goto(f"/{username}", page=page, options={"waitUntil": "networkidle0"})
+        try:
+            await page.click(username_selector)
+            await page.wait_for_selector(MAIN_WRAPPER)
+            await page.wait_for_load_state(state="networkidle")
+        except TimeoutError:
+            logger.error(f'❗️ Unexpected error. Timeout on searching user "{username}"...')
+            return []
+
         logger.debug(f"📭 Got {username} feed")
 
-        await page.waitForSelector(".video-feed-item", options={"visible": True})
+        error = await page.query_selector(ERROR_TITLE)
+
+        if error:
+            logger.info(f'😭 Error message on page: "{await error.text_content()}"')
+
+            return []
+
+        await page.wait_for_selector(USER_FEED_LIST, state="visible")
+
+        await self._paginate_feed_list(
+            page=page,
+            username=username,
+            result=result,
+            amount=amount,
+        )
+
+        await page.close()
+        return unique_dicts_by_key(result, "id")[:amount]
+
+    async def _paginate_feed_list(
+        self,
+        page: Page,
+        username: str,
+        result: List[dict],
+        amount: int,
+    ):
+        result_unique_amount: Callable = lambda: len(unique_dicts_by_key(result, "id"))
 
         pbar = tqdm(total=amount, desc=f"📈 Getting {username} feed")
-        pbar.n = min(len(result), amount)
+        pbar.n = min(result_unique_amount(), amount)
         pbar.refresh()
 
         attempts = 0
-        last_result = len(result)
+        max_attempts = 3
+        last_result = result_unique_amount()
 
-        while len(result) < amount:
+        is_attempts_limit_reached = attempts >= max_attempts
+        is_items_enough = result_unique_amount() < amount
+
+        while is_attempts_limit_reached or is_items_enough:
             logger.debug("🖱 Trying to scroll to last video item")
             await page.evaluate(
-                """
-                document.querySelector('.video-feed-item:last-child')
+                f"""
+                document.querySelector('{USER_FEED_LAST_ITEM}')
                     .scrollIntoView();
             """,
             )
-            await page.waitFor(1_000)
+            await page.wait_for_timeout(1_000)
 
-            elements = await page.JJ(".video-feed-item")
-            logger.debug(f"🔎 Found {len(elements)} items for clear")
+            elements = await page.query_selector_all(USER_FEED_ITEM)
+            logger.debug(f"🔎 Found {len(elements)} items on page by selector {USER_FEED_ITEM}")
 
-            pbar.n = min(len(result), amount)
+            pbar.n = min(result_unique_amount(), amount)
             pbar.refresh()
 
-            if last_result == len(result):
+            if last_result == result_unique_amount():
                 attempts += 1
             else:
                 attempts = 0
 
-            if attempts > 10:
+            if attempts > max_attempts:
                 pbar.clear()
-                pbar.total = len(result)
+                pbar.total = result_unique_amount()
                 logger.info(
-                    f"⚠️  After 10 attempts found {len(result)} videos. "
+                    f"⚠️  After {max_attempts} attempts found {result_unique_amount()} videos. "
                     f"Probably some videos are private",
                 )
                 break
 
-            last_result = len(result)
+            last_result = result_unique_amount()
 
-            if len(elements) < 500:
-                logger.debug("🔻 Too less for clearing page")
-                continue
+            await page.wait_for_timeout(5_000)
 
-            await page.JJeval(
-                ".video-feed-item:not(:last-child)",
-                pageFunction="(elements) => elements.forEach(el => el.remove())",
-            )
-            logger.debug(f"🎉 Cleaned {len(elements) - 1} items from page")
-            await page.waitFor(30_000)
-
-        await page.close()
         pbar.close()
-        return result[:amount]
